unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, Registry, ExtCtrls, DateUtils,
  ComCtrls, ImgList, CoolTrayIcon, IniFiles, StrUtils;

type
  TReminderForm = class(TForm)
    PopupMenu1: TPopupMenu;
    Exit1: TMenuItem;
    Timer1: TTimer;
    MessageMemo: TMemo;
    DateTimeSelectorPanel: TPanel;
    DatePicker: TDateTimePicker;
    TimePicker: TDateTimePicker;
    Panel1: TPanel;
    AddReminderButton: TButton;
    ImageList1: TImageList;
    CoolTrayIcon1: TCoolTrayIcon;
    ImageList2: TImageList;
    Open1: TMenuItem;
    DismissReminderButton: TButton;
    RemindLaterButton: TButton;
    About1: TMenuItem;
    N1: TMenuItem;
    Pending1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure AddReminderButtonClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Open1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure DismissReminderButtonClick(Sender: TObject);
    procedure RemindLaterButtonClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Pending1Click(Sender: TObject);
    function HTMLentitiesEncode(OriginalString: string): string;
    function HTMLentitiesDecode(OriginalString: string): string;
  private
    { Private declarations }
  public
    { Public declarations }
    CurrentReminderDateString: string;
  end;

const
  SECTION = 'SiliReminder';
  INI_SECTION = 'Reminders';

var
  ReminderForm: TReminderForm;
  EventsIniFile: TIniFile;

implementation

uses About, PendingReminders;

{$R *.dfm}

procedure TReminderForm.FormCreate(Sender: TObject);
begin
  EventsIniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'SiliReminder.ini');
  SetWindowLong(Application.Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
  DatePicker.DateTime := Now;
  TimePicker.DateTime := Now;
end;

procedure TReminderForm.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TReminderForm.AddReminderButtonClick(Sender: TObject);
var
  DateTimeString: string;
begin
  DateTimeToString(DateTimeString, 'yyyymmddhhnnss', Int(DatePicker.DateTime) + Frac(TimePicker.DateTime));
  EventsIniFile.DeleteKey(INI_SECTION, DateTimeString);
  EventsIniFile.WriteString(INI_SECTION, DateTimeString, HTMLentitiesEncode(Trim(MessageMemo.Lines.Text)));
  ReminderForm.Hide;
  CoolTrayIcon1.Hint := 'SiliReminder';
  MessageMemo.Clear;
end;

procedure TReminderForm.Timer1Timer(Sender: TObject);
var
  CurrentDateTimeString, AlertDateTimeString, AlertMessage: string;
  AlertDateTime: TDateTime;
  i, PendingReminders: integer;
  KeyNames: TStrings;
  DateReminderPairs: TStringList;
begin
  PendingReminders := 0;
  CoolTrayIcon1.Hint := 'SiliReminder';
  Timer1.Enabled := false;
  KeyNames := TStringList.Create;
  DateReminderPairs := TStringList.Create;
  DateReminderPairs.Sorted := True;
  try
    EventsIniFile.ReadSectionValues(INI_SECTION, KeyNames);
    DateTimeToString(CurrentDateTimeString, 'yyyymmddhhnnss', Now);
    for i := 0 to (KeyNames.Count - 1) do
      begin
        CurrentReminderDateString := copy(KeyNames[i], 1, 14);
        AlertDateTime := EncodeDateTime(
          StrToInt(copy(KeyNames[i],  1, 4)),
          StrToInt(copy(KeyNames[i],  5, 2)),
          StrToInt(copy(KeyNames[i],  7, 2)),
          StrToInt(copy(KeyNames[i],  9, 2)),
          StrToInt(copy(KeyNames[i], 11, 2)),
          StrToInt(copy(KeyNames[i], 13, 2)),
        0);
        DateTimeToString(AlertDateTimeString, 'yyyy-mm-dd hh:nn:ss am/pm', AlertDateTime);
        AlertMessage := HTMLentitiesDecode(EventsIniFile.ReadString(INI_SECTION, CurrentReminderDateString, ''));
        DateReminderPairs.Append(AlertDateTimeString+'='+AlertMessage);
        if (CurrentDateTimeString >= KeyNames[i]) then
          begin
            Inc(PendingReminders);
            CurrentReminderDateString := copy(KeyNames[i], 1, 14);
            CoolTrayIcon1.IconList := ImageList2;
            CoolTrayIcon1.CycleIcons := true;
            break;
          end;
      end;
    if (PendingReminders > 0) then
      CoolTrayIcon1.Hint := 'SiliReminder - Pending Reminders';
    Timer1.Interval := 30000;
    Timer1.Enabled := true;
  finally
    KeyNames.Free;
    if not PendingRemindersForm.Visible then
      begin
        PendingRemindersForm.ValueListEditor1.Strings.Clear;
        PendingRemindersForm.ValueListEditor1.Strings := DateReminderPairs;
      end;
    DateReminderPairs.Free;
  end;
end;

procedure TReminderForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (ReminderForm.Visible) then
    begin
      ReminderForm.Hide;
      CoolTrayIcon1.Hint := 'SiliReminder';
      Timer1.Interval := 1000;
      Timer1.Enabled := true;
      CanClose := False;
    end
  else
      Application.Terminate;
end;

procedure TReminderForm.Open1Click(Sender: TObject);
begin
  if not ReminderForm.Visible then
    begin
      DatePicker.DateTime := Now;
      TimePicker.DateTime := Now;
      ReminderForm.Show;
    end;
end;

procedure TReminderForm.FormShow(Sender: TObject);
var
  CurrentDateTimeString, AlertDateTimeString, AlertMessage: string;
  AlertDateTime: TDateTime;
  i: integer;
  KeyNames: TStrings;
  KeyName: string;
begin
  if (CurrentReminderDateString <> '') then
    begin
      CoolTrayIcon1.IconIndex := 0;
      CoolTrayIcon1.CycleIcons := false;
      DatePicker.DateTime := Now;
      TimePicker.DateTime := Now;

      KeyNames := TStringList.Create;
      try
        EventsIniFile.ReadSectionValues(INI_SECTION, KeyNames);
        DateTimeToString(CurrentDateTimeString, 'yyyymmddhhnnss', Now);
        for i := 0 to KeyNames.Count - 1 do
          begin
            if (CurrentDateTimeString >= KeyNames[i]) then
              begin
                CurrentReminderDateString := copy(KeyNames[i], 1, 14);;
                AlertDateTime := EncodeDateTime(
                  StrToInt(copy(KeyNames[i],  1, 4)),
                  StrToInt(copy(KeyNames[i],  5, 2)),
                  StrToInt(copy(KeyNames[i],  7, 2)),
                  StrToInt(copy(KeyNames[i],  9, 2)),
                  StrToInt(copy(KeyNames[i], 11, 2)),
                  StrToInt(copy(KeyNames[i], 13, 2)),
                0);
                DateTimeToString(AlertDateTimeString, 'mmm d, yyyy h:nn:ss am/pm', AlertDateTime);
                ReminderForm.Caption := AlertDateTimeString;

                DatePicker.DateTime := AlertDateTime;
                TimePicker.DateTime := AlertDateTime;
                KeyName := copy(KeyNames[i], 1, 14);
                AlertMessage := HTMLentitiesDecode(EventsIniFile.ReadString(INI_SECTION, KeyName, ''));
                MessageMemo.Lines.Clear;
                MessageMemo.Lines.Append(AlertMessage);
                AddReminderButton.Enabled := false;
                DismissReminderButton.Enabled := true;
                RemindLaterButton.Enabled := true;
                break;
              end;
          end;
      finally
        KeyNames.Free;	{ destroy the list object }
      end;
      if (CurrentReminderDateString <> '') then
        Timer1.Enabled := true;
    end;
end;

procedure TReminderForm.DismissReminderButtonClick(Sender: TObject);
begin
  EventsIniFile.DeleteKey(INI_SECTION, CurrentReminderDateString);
  ReminderForm.Hide;
  CoolTrayIcon1.Hint := 'SiliReminder';
  MessageMemo.Lines.Clear;
  AddReminderButton.Enabled := true;
  DismissReminderButton.Enabled := false;
  RemindLaterButton.Enabled := false;
  Timer1.Interval := 1000;
  Timer1.Enabled := true;
  CurrentReminderDateString := '';
  CoolTrayIcon1.Hint := 'SiliReminder';
  CoolTrayIcon1.CycleIcons := false;
  CoolTrayIcon1.IconList := ImageList1;
end;

procedure TReminderForm.RemindLaterButtonClick(Sender: TObject);
var
  DateTimeString: string;
begin
  EventsIniFile.DeleteKey(INI_SECTION, CurrentReminderDateString);
  DateTimeToString(DateTimeString, 'yyyymmddhhnnss', Int(DatePicker.DateTime) + Frac(TimePicker.DateTime));
  EventsIniFile.DeleteKey(INI_SECTION, DateTimeString);
  EventsIniFile.WriteString(INI_SECTION, DateTimeString, HTMLentitiesEncode(Trim(MessageMemo.Lines.Text)));
  ReminderForm.Hide;
  CoolTrayIcon1.Hint := 'SiliReminder';
  MessageMemo.Lines.Clear;
  AddReminderButton.Enabled := true;
  DismissReminderButton.Enabled := false;
  RemindLaterButton.Enabled := false;
  Timer1.Interval := 1000;
  Timer1.Enabled := true;
  CurrentReminderDateString := '';
  CoolTrayIcon1.Hint := 'SiliReminder';
  CoolTrayIcon1.CycleIcons := false;
  CoolTrayIcon1.IconList := ImageList1;
end;

procedure TReminderForm.About1Click(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TReminderForm.Pending1Click(Sender: TObject);
begin
  PendingRemindersForm.Show;
end;

function TReminderForm.HTMLentitiesEncode(OriginalString: string): string;
var
  EncodedString: string;
  i: integer;
begin
  EncodedString := '';
  for i := 1 to Length(OriginalString) do
    begin
      if ((ord(OriginalString[i]) >= 32) and (ord(OriginalString[i]) <= 122) and (OriginalString[i] <> '&') and (OriginalString[i] <> '=')) then
        EncodedString := EncodedString + OriginalString[i]
      else
        EncodedString := EncodedString + '&#' + IntToStr(ord(OriginalString[i])) + ';';
    end;
  Result := EncodedString;
end;

function TReminderForm.HTMLentitiesDecode(OriginalString: string): string;
var
  DecodedString: string;
  i: integer;
begin
  DecodedString := '';
  i := 1;
  while (i <= Length(OriginalString)) do
    begin
      if ((OriginalString[i] = '&') and (OriginalString[(i+1)] = '#')) then
        begin
          if (OriginalString[(i+5)] = ';') then       //&#123;
            begin
              DecodedString := DecodedString + chr(StrToInt(MidStr(OriginalString, i + 2, 3)));
              i := i + 5;
            end
          else if (OriginalString[(i+4)] = ';') then  //&#12;
            begin
              DecodedString := DecodedString + chr(StrToInt(MidStr(OriginalString, i + 2, 2)));
              i := i + 4;
            end
          else if (OriginalString[(i+3)] = ';') then  //&#1;
            begin
              DecodedString := DecodedString + chr(StrToInt(MidStr(OriginalString, i + 2, 1)));
              i := i + 3;
            end
          else
            DecodedString := DecodedString + OriginalString[i];
        end
      else
        DecodedString := DecodedString + OriginalString[i];
      i := i + 1;
    end;
  Result := DecodedString;
end;

end.
